/*
 * Copyright 2012, 2017 INFN
 *
 * Licensed under the EUPL, Version 1.2 or – as soon they
 * will be approved by the European Commission - subsequent
 * versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the
 * Licence.
 * You may obtain a copy of the Licence at:
 *
 * https://joinup.ec.europa.eu/software/page/eupl
 *
 * Unless required by applicable law or agreed to in
 * writing, software distributed under the Licence is
 * distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied.
 * See the Licence for the specific language governing
 * permissions and limitations under the Licence.
 */
#include <chaos/common/chaos_constants.h>

#include <chaos_metadata_service_client/node_monitor/ControlUnitController.h>
#include <chaos_metadata_service_client/node_monitor/ControlUnitMonitorHandler.h>

#include <boost/format.hpp>

using namespace chaos::common::data;
using namespace chaos::metadata_service_client::node_monitor;

ControlUnitController::ControlUnitController(const std::string &_node_uid):
NodeController(_node_uid),
last_seq_id(0),
cu_output_ds_key(boost::str(boost::format("%1%%2%") % getNodeUID() % chaos::DataPackPrefixID::OUTPUT_DATASET_POSTFIX)),
cu_input_ds_key(boost::str(boost::format("%1%%2%") % getNodeUID() % chaos::DataPackPrefixID::INPUT_DATASET_POSTFIX)),
cu_system_ds_key(boost::str(boost::format("%1%%2%") % getNodeUID() % chaos::DataPackPrefixID::SYSTEM_DATASET_POSTFIX)),
cu_alarm_ds_key(boost::str(boost::format("%1%%2%") % getNodeUID() % chaos::DataPackPrefixID::DEV_ALARM_DATASET_POSTFIX)),
cu_warning_ds_key(boost::str(boost::format("%1%%2%") % getNodeUID() % chaos::DataPackPrefixID::CU_ALARM_DATASET_POSTFIX)){
    monitor_key_list.push_back(cu_output_ds_key);
    monitor_key_list.push_back(cu_input_ds_key);
    monitor_key_list.push_back(cu_system_ds_key);
    monitor_key_list.push_back(cu_alarm_ds_key);
    monitor_key_list.push_back(cu_warning_ds_key);
}

ControlUnitController::~ControlUnitController() { }

void ControlUnitController::quantumSlotHasData(const std::string &key,
                                               const chaos::metadata_service_client::monitor_system::KeyValue &value) {
    bool changed = false;
    uint64_t cur_seq_id = 0;
//    if(value->hasKey(DataPackCommonKey::DPCK_SEQ_ID)) {
//        cur_seq_id = value->getUInt64Value(DataPackCommonKey::DPCK_SEQ_ID);
//        if(last_seq_id && cur_seq_id>last_seq_id) {
//            if(value->hasKey(chaos::DataPackCommonKey::DPCK_TIMESTAMP)) {
//                //improve livenes status computation
//                computeOnlineState(value->getUInt64Value(chaos::DataPackCommonKey::DPCK_TIMESTAMP));
//            }
//        }
//        last_seq_id = cur_seq_id;
//    }
    
    if (key.compare(cu_output_ds_key) == 0) {
        if((changed = CHECK_DS_CHANGED(last_ds_output, value))) {
            //update output datset key
            _updateDatsetKeyMapValue((last_ds_output = value),
                                     map_ds_out);
        }
        //call handler
        _fireUpdateDSOnHandler(DataPackCommonKey::DPCK_DATASET_TYPE_OUTPUT,
                               map_ds_out,
                               changed);
     } else if (key.compare(cu_input_ds_key) == 0) {
        if((changed = CHECK_DS_CHANGED(last_ds_input, value))) {
            //update input datset key
            _updateDatsetKeyMapValue((last_ds_input = value),
                                     map_ds_in);
        }
        //call handler
        _fireUpdateDSOnHandler(DataPackCommonKey::DPCK_DATASET_TYPE_INPUT,
                               map_ds_in,
                               changed);
    } else if (key.compare(cu_system_ds_key) == 0) {
        if((changed = CHECK_DS_CHANGED(last_ds_system, value))) {
            //update input datset key
            _updateDatsetKeyMapValue((last_ds_system = value),
                                     map_ds_sys);
        }
        //call handler
        _fireUpdateDSOnHandler(DataPackCommonKey::DPCK_DATASET_TYPE_SYSTEM,
                               map_ds_sys,
                               changed);
    } else if (key.compare(cu_alarm_ds_key) == 0) {
        if((changed = CHECK_DS_CHANGED(last_ds_alarm, value))){
            //update input datset key
            _updateDatsetKeyMapValue((last_ds_alarm = value),
                                     map_ds_alarm);
        }
        //call handler
        _fireUpdateDSOnHandler(DataPackCommonKey::DPCK_DATASET_TYPE_DEV_ALARM,
                               map_ds_alarm,
                               changed);
    } else if (key.compare(cu_warning_ds_key) == 0) {
        if((changed = CHECK_DS_CHANGED(last_ds_warning, value))){
            //update input datset key
            _updateDatsetKeyMapValue((last_ds_warning = value),
                                     map_ds_warning);
        }
        //call handler
        _fireUpdateDSOnHandler(DataPackCommonKey::DPCK_DATASET_TYPE_CU_ALARM,
                               map_ds_warning,
                               changed);
    } else {
        NodeController::quantumSlotHasData(key,
                                           value);
    }
}

void ControlUnitController::quantumSlotHasNoData(const std::string &key) {
    if (key.compare(cu_output_ds_key) == 0 ||
        key.compare(cu_input_ds_key) == 0 ||
        key.compare(cu_system_ds_key) == 0 ||
        key.compare(cu_alarm_ds_key) == 0 ||
        key.compare(cu_warning_ds_key) == 0) {
    } else {
        NodeController::quantumSlotHasNoData(key);
    }
}


void ControlUnitController::_updateDatsetKeyMapValue(chaos::metadata_service_client::monitor_system::KeyValue dataset,
                                                     MapDatasetKeyValues &map) {
    int value_type = 0;
    std::vector<std::string> key_names;
    dataset->getAllKey(key_names);
    for (std::vector<std::string>::iterator it = key_names.begin(),
         end = key_names.end();
         it != end;
         it++) {
        map[*it] = dataset->getVariantValue(*it);
    }
}

void ControlUnitController::_fireUpdateDSOnHandler(int dataset_type,
                                                   MapDatasetKeyValues &map,
                                                   bool changed) {
    ChaosLockGuard wl(list_handler_mutex);
    for (MonitoHandlerListIterator it = list_handler.begin(),
         it_end = list_handler.end();
         it != it_end;
         it++) {
        ControlUnitMonitorHandler *hndlr = dynamic_cast<ControlUnitMonitorHandler *>(*it);
        if (hndlr == NULL) {
            continue;
        }
        if(hndlr->signalOnChange() == true && changed == false) {continue;}
        
        //notify listers that online status has been changed
        hndlr->updatedDS(getNodeUID(),
                         dataset_type,
                         map);
    }
}

void ControlUnitController::_fireNoDSDataFoundOnHandler(int dataset_type) {
    ChaosLockGuard wl(list_handler_mutex);
    for (MonitoHandlerListIterator it = list_handler.begin(),
         it_end = list_handler.end();
         it != it_end;
         it++) {
        ControlUnitMonitorHandler *hndlr = dynamic_cast<ControlUnitMonitorHandler *>(*it);
        if (hndlr == NULL) {
            continue;
        }
        //notify listers that online status has been changed
        hndlr->noDSDataFound(getNodeUID(),
                             dataset_type);
    }
}

bool ControlUnitController::addHandler(NodeMonitorHandler *handler_to_add) {
    ControlUnitMonitorHandler *cu_handler = dynamic_cast<ControlUnitMonitorHandler *>(handler_to_add);
    if (cu_handler == NULL) {
        return false;
    }
    bool result = NodeController::addHandler(handler_to_add);
    if (result) {
        cu_handler->updatedDS(getNodeUID(),
                              DataPackCommonKey::DPCK_DATASET_TYPE_OUTPUT,
                              map_ds_out);
        cu_handler->updatedDS(getNodeUID(),
                              DataPackCommonKey::DPCK_DATASET_TYPE_INPUT,
                              map_ds_in);
        cu_handler->updatedDS(getNodeUID(),
                              DataPackCommonKey::DPCK_DATASET_TYPE_SYSTEM,
                              map_ds_sys);
        cu_handler->updatedDS(getNodeUID(),
                              DataPackCommonKey::DPCK_DATASET_TYPE_DEV_ALARM,
                              map_ds_alarm);
        cu_handler->updatedDS(getNodeUID(),
                              DataPackCommonKey::DPCK_DATASET_TYPE_CU_ALARM,
                              map_ds_warning);
        
    }
    return result;
}
